@@ -46,8 +46,11 @@ bijiben_new_window_internal (GApplication *app,
                              GFile *file,
                              BijiNoteObj *note_obj)
 {
-  GtkWindow *win = bjb_window_base_new();
-  BijiNoteObj* note = NULL;
+  GtkWindow *win;
+  BijiNoteObj* note;
+
+  win = bjb_window_base_new ();
+  note = NULL;
 
   if (file != NULL)
     note = biji_note_get_new_from_file (g_file_get_path(file));
@@ -60,6 +63,8 @@ bijiben_new_window_internal (GApplication *app,
 
   else
     bjb_window_base_switch_to (BJB_WINDOW_BASE (win), BJB_WINDOW_BASE_MAIN_VIEW);
+
+  gtk_widget_show_all (GTK_WIDGET (win));
 }
 
 void
@@ -179,7 +184,7 @@ go_through_notes_cb (GFileEnumerator *enumerator, GAsyncResult *res, Bijiben *se
   for (l = notes_proposal; l != NULL; l = l->next)
   {
     BijiNoteObj *note = l->data;
-    gchar *path = biji_item_get_uuid (BIJI_ITEM (note));
+    const gchar *path = biji_item_get_uuid (BIJI_ITEM (note));
 
     /* Don't add an already imported note */
     if (biji_note_book_get_item_at_path (self->priv->book, path))
@@ -204,11 +209,10 @@ go_through_notes_cb (GFileEnumerator *enumerator, GAsyncResult *res, Bijiben *se
 
       g_free (default_color);
 
-      biji_note_book_append_new_note (self->priv->book, note, FALSE);
+      biji_note_book_add_item (self->priv->book, BIJI_ITEM (note), FALSE);
       biji_note_obj_save_note (note);
     }
 
-    g_free (path);
   }
 
   /* NoteBook will notify for all opened windows */
@@ -240,10 +244,11 @@ list_notes_to_copy (GObject *src_obj, GAsyncResult *res, Bijiben *self)
   }
 }
 
-static void
-import_notes_from_x (gchar *x, Bijiben *self)
+
+void
+bijiben_import_notes (Bijiben *self, gchar *location)
 {
-  GFile *to_import = g_file_new_for_path (x);
+  GFile *to_import = g_file_new_for_path (location);
 
   g_file_enumerate_children_async (to_import, ATTRIBUTES_FOR_NOTEBOOK,
                                    G_FILE_QUERY_INFO_NONE, G_PRIORITY_DEFAULT,
@@ -252,26 +257,6 @@ import_notes_from_x (gchar *x, Bijiben *self)
   g_object_unref (to_import);
 }
 
-void
-import_notes (Bijiben *self, gchar *location)
-{
-  gchar *path_to_import;
-
-  if (g_strcmp0 (location, "tomboy") ==0)
-  {
-    path_to_import = g_build_filename (g_get_user_data_dir (), "tomboy", NULL);
-    import_notes_from_x (path_to_import, self);
-    g_free (path_to_import);
-  }
-
-  else if (g_strcmp0 (location, "gnote") ==0)
-  {
-    path_to_import = g_build_filename (g_get_user_data_dir (), "gnote", NULL);
-    import_notes_from_x (path_to_import, self);
-    g_free (path_to_import);
-  }
-}
-
 static void
 bijiben_startup (GApplication *application)
 {
