@@ -28,24 +28,37 @@
 #include "bjb-main-view.h"
 #include "bjb-window-base.h"
 
+
+/*
+ * The start-up number of items to show,
+ * and its incrementation
+ */
+
+#define BJB_ITEMS_SLICE 48
+
+
 /* Gobject */
 
 struct _BjbControllerPrivate
 {
-  BijiNoteBook  *book ;
-  gchar         *needle ;
-  gchar         *collection;
-  GtkTreeModel  *model ;
-  GtkTreeModel  *completion;
+  BijiNoteBook   *book ;
+  gchar          *needle ;
+  BijiCollection *collection;
+  GtkTreeModel   *model ;
+  GtkTreeModel   *completion;
 
-  BjbWindowBase *window;
+  BjbWindowBase  *window;
 
   /*  Private  */
   GList          *items_to_show;
-  gboolean       connected;
-  gulong         book_change;
+  gint            n_items_to_show;
+  gboolean        remaining_items;
+
+  gboolean        connected;
+  gulong          book_change;
 };
 
+
 enum {
   PROP_0,
   PROP_BOOK,
@@ -79,23 +92,22 @@ bjb_controller_init (BjbController *self)
   GtkListStore     *store ;
   GtkListStore     *completion ;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, 
-                                            BJB_TYPE_CONTROLLER, 
-                                            BjbControllerPrivate);
-  priv = self->priv ;
+  priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE
+    (self, BJB_TYPE_CONTROLLER, BjbControllerPrivate);
 
   /* Create the columns */
   store = gtk_list_store_new (GD_MAIN_COLUMN_LAST,
-                              G_TYPE_STRING,   // urn
-                              G_TYPE_STRING,   // uri
-                              G_TYPE_STRING,   // name
-                              G_TYPE_STRING,   // author
-                              GDK_TYPE_PIXBUF,   // icon then note
-                              G_TYPE_LONG, // mtime
-                              G_TYPE_BOOLEAN);   // state
+                              G_TYPE_STRING,      // urn
+                              G_TYPE_STRING,      // uri
+                              G_TYPE_STRING,      // name
+                              G_TYPE_STRING,      // author
+                              GDK_TYPE_PIXBUF,    // icon then note
+                              G_TYPE_INT64,       // mtime
+                              G_TYPE_BOOLEAN);    // state
 
   priv->model = GTK_TREE_MODEL(store) ;
   priv->items_to_show = NULL;
+  priv->n_items_to_show = BJB_ITEMS_SLICE;
   priv->needle = NULL;
   priv->collection = NULL;
   priv->connected = FALSE;
@@ -182,20 +194,72 @@ bjb_controller_set_property (GObject  *object,
   }
 }
 
+/* get iter at this item
+ * or get iter at first note if item == NULL*/
+static gboolean
+bjb_controller_get_iter (BjbController *self,
+                         BijiItem *item,
+                         GtkTreeIter **iter)
+{
+  BjbControllerPrivate *priv = self->priv;
+  gboolean retval = FALSE;
+  gboolean try;
+  const gchar *needle = NULL;
+
+  if (item && BIJI_IS_ITEM (item))
+      needle = biji_item_get_uuid (item);
+
+  try = gtk_tree_model_get_iter_first (priv->model, *iter);
+
+  if (!try)
+    *iter = NULL;
+
+  while (try)
+  {
+    gchar *item_path;
+    gtk_tree_model_get (priv->model, *iter, GD_MAIN_COLUMN_URI, &item_path,-1);
+
+    /* If we look for the item, check by uid */
+    if (needle && g_strcmp0 (item_path, needle) == 0)
+      retval = TRUE;
+
+    /* Else we check for the first note */
+    else if (!needle && BIJI_IS_NOTE_OBJ (
+              biji_note_book_get_item_at_path (self->priv->book, item_path)))
+      retval = TRUE;
+
+    g_free (item_path);
+
+    if (retval)
+      break;
+
+    else
+      try = gtk_tree_model_iter_next (priv->model, *iter);
+  }
+
+  return retval;
+}
+
+/* Either append, prepend, or, if iter provider,
+ * prepend just before this iter */
 static void
 bjb_controller_add_item (BjbController *self,
                          BijiItem      *item,
-                         gboolean       prepend)
+                         gboolean       prepend,
+                         GtkTreeIter   *sibling)
 {
   GtkTreeIter    iter;
   GtkListStore  *store;
   GdkPixbuf     *pix = NULL;
-  gchar         *uuid;
+  const gchar   *uuid;
 
   g_return_if_fail (BIJI_IS_ITEM (item));
   store = GTK_LIST_STORE (self->priv->model);
 
-  if (prepend)
+  if (sibling)
+    gtk_list_store_insert_before (store, &iter, sibling);
+
+  else if (prepend)
     gtk_list_store_insert (store, &iter, 0);
 
   else
@@ -228,68 +292,58 @@ bjb_controller_add_item (BjbController *self,
        GD_MAIN_COLUMN_PRIMARY_TEXT, biji_item_get_title (item),
        GD_MAIN_COLUMN_SECONDARY_TEXT, NULL,
        GD_MAIN_COLUMN_ICON, pix,
-       GD_MAIN_COLUMN_MTIME, biji_item_get_last_change (item),
+       GD_MAIN_COLUMN_MTIME, biji_item_get_mtime (item),
        GD_MAIN_COLUMN_SELECTED, FALSE,
        -1);
 
-  g_free (uuid);
-
 }
 
 /* If the user searches for notes, is the note searched? */
 static void
-bjb_controller_add_note_if_needed (BjbController *self,
-                                   BijiNoteObj   *note,
-                                   gboolean       prepend)
+bjb_controller_add_item_if_needed (BjbController *self,
+                                   BijiItem      *item,
+                                   gboolean       prepend,
+                                   GtkTreeIter   *sibling)
 {
-  gboolean need_to_add_note =FALSE;
-  gchar *title, *content;
-  GList *collections, *l;
+  gboolean need_to_add_item = FALSE;
+  gchar *content;
+  const gchar *title;
   BjbControllerPrivate *priv = self->priv;
-  BijiItem *item = BIJI_ITEM (note);
 
   /* No note... */
-  if (!note || !BIJI_IS_NOTE_OBJ (note))
+  if (!item || !BIJI_IS_ITEM (item))
     return;
 
   /* No search - we add the note */
   if (!priv->needle || g_strcmp0 (priv->needle, "")==0)
   {
-    need_to_add_note = TRUE;
+    if (!priv->collection)
+      need_to_add_item = TRUE;
+
+    /* To do : we might have a collection
+     * but have to udpate the view */
   }
 
   /* a search.. we test...*/
   else
   {
-
     title = biji_item_get_title (item);
-    content = biji_note_get_raw_text (note);
 
-    /* matching title or content ... */
-    if (g_strrstr (title  , priv->needle) != NULL ||
-        g_strrstr (content, priv->needle) != NULL  )
-      need_to_add_note = TRUE;
+    /* matching title... */
+    if (g_strrstr (title, priv->needle) != NULL)
+      need_to_add_item = TRUE;
 
-    /* last chance, matching collections... */
-    else
+    /* matching content */
+    else if (BIJI_IS_NOTE_OBJ (item))
     {
-      collections = biji_note_obj_get_collections (note);
-    
-      for (l = collections; l != NULL; l=l->next)
-      {
-        if (g_strrstr (l->data, title))
-        {
-          need_to_add_note = TRUE;
-          break;
-        }
-      }
-
-      g_list_free (collections);
+      content = biji_note_get_raw_text (BIJI_NOTE_OBJ (item));
+      if (g_strrstr (content, priv->needle) != NULL)
+        need_to_add_item = TRUE;
     }
   }
 
-  if (need_to_add_note)
-    bjb_controller_add_item (self, item, prepend);
+  if (need_to_add_item)
+    bjb_controller_add_item (self, item, prepend, sibling);
 }
 
 void
@@ -306,7 +360,7 @@ bjb_controller_update_view (BjbController *self)
 
   for (l = items; l != NULL; l = l->next)
   {
-    bjb_controller_add_item (self, l->data, FALSE);
+    bjb_controller_add_item (self, l->data, FALSE, NULL);
   }
 }
 
@@ -333,41 +387,88 @@ most_recent_item_first (gconstpointer a, gconstpointer b)
    * two collections, use the most recent cookbook */
   else
   {
-    result =   biji_item_get_last_change (other)
-             - biji_item_get_last_change (one);
+    result =   biji_item_get_mtime (other)
+             - biji_item_get_mtime (one);
   }
 
   return result;
 }
 
+
+
 static void
-sort_items (BjbController *self)
+sort_items (GList **to_show)
 {
-  self->priv->items_to_show = g_list_sort (self->priv->items_to_show,
-                                           most_recent_item_first);
+  *to_show = g_list_sort (*to_show, most_recent_item_first);
+}
+
+
+static void
+notify_displayed_items_changed (BjbController *self)
+{
+  /*gboolean shown;
+
+  shown = (self->priv->items_to_show != NULL);*/
+  g_signal_emit (G_OBJECT (self),
+                 bjb_controller_signals[DISPLAY_NOTES_CHANGED],
+                 0,
+                 //shown,
+                 (self->priv->items_to_show != NULL),
+                 self->priv->remaining_items);  
 }
 
 static void
-sort_and_update (BjbController *self)
+update (BjbController *self)
 {
-  sort_items (self);
+  bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_MAIN_VIEW);
   bjb_controller_update_view (self);
 
-  g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
+  notify_displayed_items_changed (self);
 }
 
+
 static void
-update_controller_callback (GObject *source_object,
-                            GAsyncResult *res,
+update_controller_callback (GList *result,
                             gpointer user_data)
 {
-  GList *result;
-  BjbController *self = BJB_CONTROLLER (user_data);
+  BjbController *self;
+  BjbControllerPrivate *priv;
+  GList *l;
+  gint i;
 
-  result = biji_get_notes_with_strings_or_collection_finish (source_object, res, self->priv->book);
-  self->priv->items_to_show = result;
+  self = BJB_CONTROLLER (user_data);
+  priv = self->priv;
+  priv->remaining_items = FALSE;
 
-  sort_and_update (self);
+  if (!result)
+  {
+    bjb_window_base_switch_to (priv->window, BJB_WINDOW_BASE_NO_RESULT);
+    return;
+  }
+
+  sort_items (&result);
+  i = 0;
+
+  /* We are supposed to show n items. While not reach,
+   * add to the list. As soon as reached, keep in mind that */
+
+  for (l=result ; l!= NULL ; l=l->next)
+  {
+    if (i< priv->n_items_to_show)
+    {
+      priv->items_to_show = g_list_prepend (priv->items_to_show, l->data);
+      i ++;
+    }
+
+    else if (l->next != NULL)
+    {
+      priv->remaining_items = TRUE;
+      break;
+    }
+  }
+
+  priv->items_to_show = g_list_reverse (priv->items_to_show);
+  update (self);
 }
 
 void
@@ -375,32 +476,30 @@ bjb_controller_apply_needle (BjbController *self)
 {
   BjbControllerPrivate *priv = self->priv;
   gchar *needle;
+  GList *all_notes;
 
-  if (priv->items_to_show)
-    g_clear_pointer (&priv->items_to_show, g_list_free);
-  
   needle = priv->needle;
+  g_clear_pointer (&priv->items_to_show, g_list_free);
 
   /* Show all notes */
   if (needle == NULL || g_strcmp0 (needle,"") == 0)
   {
-    priv->items_to_show = biji_note_book_get_items (self->priv->book);
+    all_notes = biji_note_book_get_items (self->priv->book);
 
     /* If there are no note, report this */
-    if (!priv->items_to_show)
-      bjb_window_base_switch_to (self->priv->window,
-                                 BJB_WINDOW_BASE_NO_NOTE);
+    if (all_notes == NULL)
+    {
+      bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_NO_NOTE);
+      return;
+    }
 
     /* Otherwise do show existing notes */
-    else
-      sort_and_update (self);
-
+    update_controller_callback (all_notes, self);
     return;
   }
 
-  /* There is a research, apply lookup
-   * TODO : also look for collections */
-  biji_get_notes_with_string_or_collection_async (needle, update_controller_callback, self);
+  /* There is a research, apply lookup */
+  biji_get_items_matching_async (self->priv->book, needle, update_controller_callback, self);
 }
 
 static void
@@ -442,35 +541,6 @@ refresh_completion(BjbController *self)
   }
 }
 
-static gboolean
-bjb_controller_get_iter_at_item (BjbController *self, BijiItem *item, GtkTreeIter **iter)
-{
-  BjbControllerPrivate *priv = self->priv;
-  gchar *needle = biji_item_get_uuid (item);
-  gboolean retval = FALSE;
-  gboolean still = gtk_tree_model_get_iter_first (priv->model, *iter);
-
-  while (still)
-  {
-    gchar *item_path;
-
-    gtk_tree_model_get (priv->model, *iter, GD_MAIN_COLUMN_URI, &item_path,-1);
-
-    if (g_strcmp0 (item_path, needle)==0)
-      retval = TRUE;
-
-    g_free (item_path);
-
-    if (retval)
-      break;
-
-    else
-      still = gtk_tree_model_iter_next (priv->model, *iter);
-  }
-
-  g_free (needle);
-  return retval;
-}
 
 /* Depending on the change at data level,
  * the view has to be totaly refreshed or just amended */
@@ -488,48 +558,56 @@ on_book_changed (BijiNoteBook           *book,
 
   switch (flag)
   {
-    /* If this is a *new* note, per def prepend
-     * But do not add a new note to a search window */
+    /* If this is a *new* item, per def prepend */
     case BIJI_BOOK_ITEM_ADDED:
-        /* Todo : handle collection as well */
-        if (BIJI_IS_NOTE_OBJ (item))
-        {
-          note = BIJI_NOTE_OBJ (item);
-          bjb_controller_add_note_if_needed (self, note, TRUE);
+          if (BIJI_IS_NOTE_OBJ (item))
+            bjb_controller_get_iter (self, NULL, &p_iter);
+
+          else if (BIJI_IS_COLLECTION (item))
+            p_iter = NULL;
+
+          bjb_controller_add_item_if_needed (self, item, TRUE, p_iter);
+          priv->n_items_to_show ++;
           priv->items_to_show = g_list_prepend (priv->items_to_show, note);
-          g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
-        }
+          notify_displayed_items_changed (self);
       break;
 
-    /* FIXME - a note has to be added after the collections... */
+    /* Same comment, prepend but collection before note */
     case BIJI_BOOK_NOTE_AMENDED:
-      if (bjb_controller_get_iter_at_item (self, item, &p_iter))
+      if (bjb_controller_get_iter (self, item, &p_iter))
       {
         gtk_list_store_remove (GTK_LIST_STORE (priv->model), p_iter);
-        bjb_controller_add_note_if_needed (self, note, TRUE);
+
+        if (BIJI_IS_NOTE_OBJ (item))
+          bjb_controller_get_iter (self, NULL, &p_iter);
+
+        else if (BIJI_IS_COLLECTION (item))
+          p_iter = NULL;
+
+        bjb_controller_add_item_if_needed (self, item, TRUE, p_iter);
       }
       break;
 
     /* If color changed we just amend the icon */
-    case BIJI_BOOK_NOTE_COLORED:
+    case BIJI_BOOK_ITEM_ICON_CHANGED:
       if (bjb_main_view_get_view_type (
              bjb_window_base_get_main_view (self->priv->window)) == GD_MAIN_VIEW_ICON
-          && bjb_controller_get_iter_at_item (self, item, &p_iter))
+          && bjb_controller_get_iter (self, item, &p_iter))
         gtk_list_store_set (GTK_LIST_STORE (priv->model), p_iter,
                             GD_MAIN_COLUMN_ICON, biji_item_get_icon (item), -1);
       else if (bjb_main_view_get_view_type (
              bjb_window_base_get_main_view (self->priv->window)) == GD_MAIN_VIEW_LIST
-          && bjb_controller_get_iter_at_item (self, item, &p_iter))
+          && bjb_controller_get_iter (self, item, &p_iter))
         gtk_list_store_set (GTK_LIST_STORE (priv->model), p_iter,
                             GD_MAIN_COLUMN_ICON, biji_item_get_emblem (item), -1);
       break;
 
     case BIJI_BOOK_ITEM_TRASHED:
-      if (bjb_controller_get_iter_at_item (self, item, &p_iter))
+      if (bjb_controller_get_iter (self, item, &p_iter))
         gtk_list_store_remove (GTK_LIST_STORE (priv->model), p_iter);
 
       priv->items_to_show = g_list_remove (priv->items_to_show, note);
-      g_signal_emit (G_OBJECT (self), bjb_controller_signals[DISPLAY_NOTES_CHANGED],0);
+      notify_displayed_items_changed (self);
       break;
 
     default:
@@ -540,7 +618,7 @@ on_book_changed (BijiNoteBook           *book,
   refresh_completion(self);
 }
 
-void
+static void
 bjb_controller_connect (BjbController *self)
 {
   BjbControllerPrivate *priv = self->priv;
@@ -568,6 +646,8 @@ static void
 bjb_controller_constructed (GObject *obj)
 {
   G_OBJECT_CLASS(bjb_controller_parent_class)->constructed(obj);
+
+  bjb_controller_connect (BJB_CONTROLLER (obj));
 }
 
 static void
@@ -592,15 +672,17 @@ bjb_controller_class_init (BjbControllerClass *klass)
                                                   G_TYPE_NONE,
                                                   0);
 
-  bjb_controller_signals[DISPLAY_NOTES_CHANGED] = g_signal_new ( "display-notes-changed" ,
+  bjb_controller_signals[DISPLAY_NOTES_CHANGED] = g_signal_new ( "display-items-changed" ,
                                                   G_OBJECT_CLASS_TYPE (klass),
                                                   G_SIGNAL_RUN_LAST,
                                                   0, 
                                                   NULL, 
                                                   NULL,
-                                                  g_cclosure_marshal_VOID__BOOLEAN,
+                                                  _biji_marshal_VOID__BOOLEAN_BOOLEAN,
                                                   G_TYPE_NONE,
-                                                  0);
+                                                  2,
+                                                  G_TYPE_BOOLEAN,
+                                                  G_TYPE_BOOLEAN);
 
   properties[PROP_BOOK] = g_param_spec_object ("book",
                                                "Book",
@@ -691,66 +773,60 @@ bjb_controller_get_completion(BjbController *self)
   return self->priv->completion ;
 }
 
+
 gboolean
 bjb_controller_shows_item (BjbController *self)
 {
-  if (self->priv->items_to_show)
-    return TRUE;
-
-  return FALSE;
+  return (self->priv->items_to_show != NULL);
 }
 
-static void
-bjb_controller_show_collection (GObject *source_object,
-                                GAsyncResult *res,
-                                gpointer user_data)
-{
-  GList *result;
-  BjbController *self = BJB_CONTROLLER (user_data);
-
-  result = biji_get_items_with_collection_finish (source_object, res, self->priv->book);
-  self->priv->items_to_show = result;
-
-  sort_and_update (self);
-  bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_MAIN_VIEW);
-}
-
-gchar *
+BijiCollection *
 bjb_controller_get_collection (BjbController *self)
 {
   return self->priv->collection;
 }
 
+
 void
 bjb_controller_set_collection (BjbController *self,
-                               gchar         *to_open)
+                               BijiCollection *coll)
 {
   /* Going back from a collection */
-  if (!to_open)
+  if (!coll)
   {
     if (!self->priv->collection)
       return;
 
     bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_SPINNER_VIEW);
-    g_clear_pointer (&self->priv->collection, g_free);
+    self->priv->collection = NULL;
     bjb_controller_apply_needle (self);
-    bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_MAIN_VIEW);
     return;
   }
 
   /* Opening an __existing__ collection */
   bjb_window_base_switch_to (self->priv->window, BJB_WINDOW_BASE_SPINNER_VIEW);
-  g_list_free (self->priv->items_to_show);
-
-  if (self->priv->needle)
-    g_free (self->priv->needle);
-
-  if (self->priv->collection)
-    g_free (self->priv->collection);
+  g_clear_pointer (&self->priv->items_to_show, g_list_free);
+  g_clear_pointer (&self->priv->needle, g_free);
 
   self->priv->needle = g_strdup ("");
-  self->priv->collection = g_strdup (to_open);
-  biji_get_items_with_collection_async (to_open,
-                                        bjb_controller_show_collection,
+  self->priv->collection = coll;
+  biji_get_items_with_collection_async (self->priv->book,
+                                        biji_item_get_title (BIJI_ITEM (coll)),
+                                        update_controller_callback,
                                         self);
 }
+
+
+void
+bjb_controller_show_more (BjbController *self)
+{
+  self->priv->n_items_to_show += BJB_ITEMS_SLICE;
+
+  /* FIXME: this method to refresh is just non sense */
+
+  if (self->priv->collection != NULL)
+    bjb_controller_set_collection (self, self->priv->collection);
+
+  else
+    on_needle_changed (self);
+}
