@@ -221,7 +221,8 @@ bjb_editor_toolbar_init (BjbEditorToolbar *self)
   gtk_image_set_pixel_size (GTK_IMAGE (image), 24);
 
   gtk_container_add (GTK_CONTAINER (priv->toolbar_link), image);
-  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->toolbar_link), _("Link"));
+  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->toolbar_link),
+                               _("Copy selection to a new note"));
   gtk_container_add (GTK_CONTAINER (priv->box), priv->toolbar_link);
   gtk_widget_override_background_color (priv->toolbar_link,
                                         GTK_STATE_FLAG_NORMAL,
@@ -422,7 +423,7 @@ link_callback (GtkWidget *button, BjbEditorToolbar *self)
   window = bjb_note_view_get_base_window (priv->view);
   book = bjb_window_base_get_book(window);
 
-  result = biji_note_book_get_new_note_from_string (book, link);
+  result = biji_note_book_note_new (book, link);
 
   /* Change result color. */
   if (biji_note_obj_get_rgba (priv->note, &color))
