@@ -62,6 +62,7 @@ struct _BjbMainToolbarPrivate
 
   /* When note view */
   BijiNoteObj      *note;
+  GtkWidget        *share;
   gulong            note_renamed;
   gulong            note_color_changed;
   GtkAccelGroup    *accel;
@@ -92,13 +93,13 @@ on_new_note_clicked (GtkWidget *but, BjbMainView *view)
 
   /* append note to collection */
   book = bjb_window_base_get_book (bjb_main_view_get_window (view));
-  result = biji_note_book_get_new_note_from_string (book, "");
+  result = biji_note_book_note_new (book, NULL);
 
   /* Go to that note */
   switch_to_note_view(view,result);
 }
 
-static void populate_main_toolbar(BjbMainToolbar *self);
+static void populate_main_toolbar (BjbMainToolbar *self);
 
 static gboolean
 update_selection_label (BjbMainToolbar *self)
@@ -107,7 +108,7 @@ update_selection_label (BjbMainToolbar *self)
   gint length;
   gchar *label;
 
-  selected = bjb_main_view_get_selection (self->priv->parent);
+  selected = bjb_main_view_get_selected_items (self->priv->parent);
   length = g_list_length (selected);
   g_list_free (selected);
 
@@ -140,6 +141,8 @@ on_view_selection_changed_cb (BjbMainToolbar *self)
   gtk_widget_reset_style (widget);
   populate_main_toolbar (self);
 
+  /* If we were already on selection mode,
+   * the bar is not totaly refreshed. just udpate label */
   if (self->priv->type == BJB_TOOLBAR_SELECT)
     update_selection_label (self);
 
@@ -150,10 +153,17 @@ static void
 on_selection_mode_clicked (GtkWidget *button, BjbMainToolbar *self)
 {
   if (bjb_main_view_get_selection_mode (self->priv->parent))
+  {
     bjb_main_view_set_selection_mode (self->priv->parent, FALSE);
+  }
 
+  /* Force refresh. We go to selection mode but nothing yet selected
+   * Thus no signal emited */
   else
+  {
     bjb_main_view_set_selection_mode (self->priv->parent, TRUE);
+    on_view_selection_changed_cb (self);
+  }
 }
 
 static gboolean
@@ -202,48 +212,49 @@ on_button_press (GtkWidget* widget,
 static void
 on_search_button_clicked (BjbMainToolbarPrivate *priv)
 {
-  BjbSearchToolbar *bar;
+  gboolean show_bar;
 
-  bar = bjb_main_view_get_search_toolbar (priv->parent);
-
-  if (bar == NULL)
-    return;
-
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->search)))
-    bjb_search_toolbar_fade_in (bar);
-
-  else
-    bjb_search_toolbar_fade_out (bar);
+  show_bar = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->search));
+  bjb_window_base_set_show_search_bar (BJB_WINDOW_BASE (priv->window), show_bar);
 }
 
 static void
 add_search_button (BjbMainToolbar *self)
 {
   BjbMainToolbarPrivate *priv = self->priv;
+  gboolean active;
 
   priv->search = gd_main_toolbar_add_toggle (GD_MAIN_TOOLBAR (self),
                                              "edit-find-symbolic",
                                              NULL,
                                              FALSE);
+  gtk_widget_set_tooltip_text (priv->search,
+                               _("Search note titles, content and collections"));
+
+  active =  bjb_window_base_get_show_search_bar (BJB_WINDOW_BASE (self->priv->window));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->search), active);
 
   g_signal_connect_swapped (priv->search, "clicked",
                             G_CALLBACK (on_search_button_clicked), priv);
 }
 
-static void
-update_selection_buttons (BjbMainToolbarPrivate *priv)
-{
-  gboolean some_note_is_visible = bjb_controller_shows_item (priv->controller);
 
+static void
+update_selection_buttons (BjbController *controller,
+                          gboolean some_item_is_visible,
+                          gboolean remaining,
+                          BjbMainToolbarPrivate *priv)
+{
   if (priv->grid)
-    gtk_widget_set_sensitive (priv->grid, some_note_is_visible);
+    gtk_widget_set_sensitive (priv->grid, some_item_is_visible);
 
   if (priv->list)
-    gtk_widget_set_sensitive (priv->list, some_note_is_visible);
+    gtk_widget_set_sensitive (priv->list, some_item_is_visible);
 
-  gtk_widget_set_sensitive (priv->select, some_note_is_visible);
+  gtk_widget_set_sensitive (priv->select, some_item_is_visible);
 }
 
+
 static void
 populate_bar_for_selection (BjbMainToolbar *self)
 {
@@ -255,7 +266,8 @@ populate_bar_for_selection (BjbMainToolbar *self)
 
   /* Select */
   priv->select = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (self),
-                                             NULL,"Done", FALSE);
+                                             NULL, _("Done"), FALSE);
+  gtk_widget_set_tooltip_text (priv->select, _("Exit selection mode"));
   context = gtk_widget_get_style_context (priv->select);
   gtk_style_context_add_class (context, "suggested-action");
   gtk_widget_reset_style (priv->select);
@@ -277,13 +289,14 @@ static void
 update_label_for_standard (BjbMainToolbar *self)
 {
   BjbMainToolbarPrivate *priv = self->priv;
-  gchar *collection, *needle, *label;
+  BijiCollection *coll;
+  gchar *needle, *label;
 
-  collection = bjb_controller_get_collection (priv->controller);
+  coll = bjb_controller_get_collection (priv->controller);
   needle = bjb_controller_get_needle (priv->controller);
 
-  if (collection)
-    label = g_strdup_printf ("%s", collection);
+  if (coll)
+    label = g_strdup_printf ("%s", biji_item_get_title (BIJI_ITEM (coll)));
 
   else if (needle && g_strcmp0 (needle, "") !=0)
     label = g_strdup_printf (_("Results for %s"), needle);
@@ -294,10 +307,10 @@ update_label_for_standard (BjbMainToolbar *self)
   gd_main_toolbar_set_labels (GD_MAIN_TOOLBAR (self), label, NULL);
   g_free (label);
 
-  self->priv->display_notes = g_signal_connect_swapped (self->priv->controller,
-                                                        "display-notes-changed",
-                                                        G_CALLBACK (update_selection_buttons),
-                                                        self->priv);
+  self->priv->display_notes = g_signal_connect (self->priv->controller,
+                                                "display-items-changed",
+                                                G_CALLBACK (update_selection_buttons),
+                                                self->priv);
 }
 
 static void
@@ -324,7 +337,7 @@ populate_bar_for_standard(BjbMainToolbar *self)
 {
   BjbMainToolbarPrivate *priv = self->priv;
   GtkWidget *bin = NULL;
-  gchar *collection;
+  BijiCollection *coll;
 
   /* Label */
   update_label_for_standard (self);
@@ -332,9 +345,9 @@ populate_bar_for_standard(BjbMainToolbar *self)
          "search-changed", G_CALLBACK(update_label_for_standard), self);
 
   /* Go back to all notes */
-  collection = bjb_controller_get_collection (priv->controller);
+  coll = bjb_controller_get_collection (priv->controller);
 
-  if (collection)
+  if (coll)
   {
     GtkWidget *back, *button;
 
@@ -372,6 +385,7 @@ populate_bar_for_standard(BjbMainToolbar *self)
                                             "object-select-symbolic",
                                             NULL,
                                             FALSE);
+  gtk_widget_set_tooltip_text (priv->select, _("Selection mode"));
 
   g_signal_connect (priv->select,"clicked",
                     G_CALLBACK(on_selection_mode_clicked),self);
@@ -391,6 +405,8 @@ populate_bar_for_icon_view(BjbMainToolbar *self)
                                          "view-list-symbolic",
                                          NULL,
                                          FALSE);
+  gtk_widget_set_tooltip_text (priv->list,
+                               _("View notes and collections in a list"));
 
   g_signal_connect (priv->list, "clicked",
                     G_CALLBACK(on_view_mode_clicked),self);
@@ -409,6 +425,8 @@ populate_bar_for_list_view(BjbMainToolbar *self)
                                           "view-grid-symbolic",
                                           NULL,
                                           FALSE);
+  gtk_widget_set_tooltip_text (priv->grid,
+                               _("View notes and collections in a grid"));
 
   g_signal_connect (priv->grid, "clicked",
                     G_CALLBACK(on_view_mode_clicked),self);
@@ -481,6 +499,21 @@ on_note_color_changed (BijiNoteObj *note, GtkColorButton *button)
     gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (button), &color);
 }
 
+static void
+on_note_content_changed (BjbMainToolbar *self)
+{
+  gchar *str = NULL;
+  gboolean sensitive = TRUE;
+
+  if (self->priv->note)
+    str = biji_note_get_raw_text (self->priv->note);
+
+  if (!str || g_strcmp0 (str, "") == 0 || g_strcmp0 (str, "\n") == 0)
+    sensitive = FALSE;
+
+  gtk_widget_set_sensitive (self->priv->share, sensitive);
+}
+
 static void
 bjb_toggle_bullets (BijiWebkitEditor *editor)
 {
@@ -579,7 +612,7 @@ bjb_note_menu_new (BjbMainToolbar *self)
   gtk_widget_show (item);
 
   /* Ordered list as 1.mouse 2.cats 3.dogs */
-  item = gtk_menu_item_new_with_label (_("List"));
+  item = gtk_menu_item_new_with_label (_("Numbered List"));
   gtk_menu_shell_append (GTK_MENU_SHELL (result), item);
   g_signal_connect_swapped (item, "activate",
                             G_CALLBACK (bjb_toggle_list), editor);
@@ -596,7 +629,7 @@ bjb_note_menu_new (BjbMainToolbar *self)
                    G_CALLBACK(action_rename_note_callback),self);
   gtk_widget_show(item);
 
-  item = gtk_menu_item_new_with_label(_("Tags"));
+  item = gtk_menu_item_new_with_label(_("Collections"));
   gtk_menu_shell_append(GTK_MENU_SHELL(result),item);
   g_signal_connect(item,"activate",
                    G_CALLBACK(action_view_tags_callback),self);
@@ -671,6 +704,7 @@ populate_bar_for_note_view (BjbMainToolbar *self)
   }
 
   button = bjb_color_button_new ();
+  gtk_widget_set_tooltip_text (button, _("Note color"));
   gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (button), &color);
 
   gd_main_toolbar_add_widget (bar, button, FALSE);
@@ -684,26 +718,39 @@ populate_bar_for_note_view (BjbMainToolbar *self)
                              G_CALLBACK (on_note_color_changed), button);
 
   /* Sharing */
-  button = gd_main_toolbar_add_button (bar, "send-to-symbolic",
-                                       NULL, FALSE);
+  priv->share = gd_main_toolbar_add_button (bar, "send-to-symbolic",
+                                           NULL, FALSE);
+  gtk_widget_set_tooltip_text (priv->share, _("Share note"));
 
-  g_signal_connect (button, "clicked",
+  g_signal_connect (priv->share, "clicked",
                     G_CALLBACK (on_email_note_callback), priv->note);
 
+  g_signal_connect_swapped (biji_note_obj_get_editor (priv->note),
+                            "user-changed-contents",
+                            G_CALLBACK (on_note_content_changed),
+                            self);
+
+  on_note_content_changed (self);
+
   /* Menu */
   button = gd_main_toolbar_add_menu (bar,
                                      "emblem-system-symbolic",
                                      NULL,
                                      FALSE);
+  gtk_widget_set_tooltip_text (button, _("More options…"));
 
   gtk_menu_button_set_popup (GTK_MENU_BUTTON (button),
                              bjb_note_menu_new (self));
 }
 
 static void
-populate_bar_switch(BjbMainToolbar *self)
+populate_bar_switch (BjbMainToolbar *self)
 {
-  switch (self->priv->type)
+  BjbMainToolbarPrivate *priv;
+
+  priv = self->priv;
+
+  switch (priv->type)
   {
     case BJB_TOOLBAR_SELECT:
       populate_bar_for_selection (self);
@@ -712,13 +759,19 @@ populate_bar_switch(BjbMainToolbar *self)
     case BJB_TOOLBAR_STD_ICON:
       add_search_button (self);
       populate_bar_for_icon_view(self);
-      update_selection_buttons (self->priv);
+      update_selection_buttons (priv->controller,
+                                bjb_controller_shows_item (priv->controller),
+                                TRUE,
+                                self->priv);
       break;
 
     case BJB_TOOLBAR_STD_LIST:
       add_search_button (self);
       populate_bar_for_list_view(self);
-      update_selection_buttons (self->priv);
+      update_selection_buttons (priv->controller,
+                                bjb_controller_shows_item (priv->controller),
+                                TRUE,
+                                self->priv);
       break;
 
     case BJB_TOOLBAR_NOTE_VIEW:
@@ -742,24 +795,33 @@ populate_main_toolbar(BjbMainToolbar *self)
 
   view_type = bjb_window_base_get_view_type (BJB_WINDOW_BASE (priv->window));
 
-  if (view_type == BJB_WINDOW_BASE_NOTE_VIEW)
-    to_be = BJB_TOOLBAR_NOTE_VIEW;
-
-  else if (view_type == BJB_WINDOW_BASE_MAIN_VIEW)
+  switch (view_type)
   {
-    if (bjb_main_view_get_selection_mode (priv->parent) == TRUE)
-      to_be = BJB_TOOLBAR_SELECT;
+    case BJB_WINDOW_BASE_NOTE_VIEW:
+      to_be = BJB_TOOLBAR_NOTE_VIEW;
+      break;
 
-    if (bjb_main_view_get_view_type (priv->parent) == GD_MAIN_VIEW_ICON)
-      to_be = BJB_TOOLBAR_STD_ICON;
+    case BJB_WINDOW_BASE_NO_NOTE:
+    case BJB_WINDOW_BASE_NO_RESULT:
+    case BJB_WINDOW_BASE_MAIN_VIEW:
 
-    else if (bjb_main_view_get_view_type (priv->parent) == GD_MAIN_VIEW_LIST)
-      to_be = BJB_TOOLBAR_STD_LIST;
+      if (bjb_main_view_get_selection_mode (priv->parent) == TRUE)
+        to_be = BJB_TOOLBAR_SELECT;
+
+      else if (bjb_main_view_get_view_type (priv->parent) == GD_MAIN_VIEW_ICON)
+        to_be = BJB_TOOLBAR_STD_ICON;
+
+      else if (bjb_main_view_get_view_type (priv->parent) == GD_MAIN_VIEW_LIST)
+        to_be = BJB_TOOLBAR_STD_LIST;
+
+      break;
+
+    /* Not really a toolbar,
+     * still used for Spinner */
+    default:
+      to_be = BJB_TOOLBAR_0;
   }
 
-  else
-    to_be = BJB_TOOLBAR_0;
-
   /* Simply clear then populate */
   if (to_be != priv->type)
   {
