@@ -25,6 +25,7 @@
 #include "bjb-app-menu.h"
 #include "bjb-bijiben.h"
 #include "bjb-controller.h"
+#include "bjb-load-more-button.h"
 #include "bjb-main-toolbar.h"
 #include "bjb-main-view.h"
 #include "bjb-note-tag-dialog.h"
@@ -65,12 +66,10 @@ struct _BjbMainViewPriv {
   /* Selection Mode */
   BjbSelectionToolbar  *select_bar;
 
-  /* Search Entry  */
-  BjbSearchToolbar *search_bar;
-
   /* View Notes , model */
-  GdMainView       *view ; 
-  BjbController    *controller ;
+  GdMainView       *view ;
+  BjbController    *controller;
+  GtkWidget        *load_more;
 
   /* Signals */
   gulong key;
@@ -182,7 +181,6 @@ biji_main_view_constructor (GType                  gtype,
 void
 switch_to_note_view (BjbMainView *self, BijiNoteObj *note)
 {
-  bjb_search_toolbar_disconnect (self->priv->search_bar);
   bjb_main_view_disconnect_handlers (self);
   bjb_window_base_switch_to_note (BJB_WINDOW_BASE (self->priv->window), note);
 }
@@ -201,7 +199,7 @@ show_window_if_note (gpointer data, gpointer user_data)
 }
 
 static void
-switch_to_note (BjbMainView *view, BijiItem *to_open)
+switch_to_item (BjbMainView *view, BijiItem *to_open)
 {
   if (BIJI_IS_NOTE_OBJ (to_open))
   {
@@ -221,10 +219,10 @@ switch_to_note (BjbMainView *view, BijiItem *to_open)
 
   /* Collection
    * TODO : check if already opened (same as above) */
-  else
+  else if (BIJI_IS_COLLECTION (to_open))
   {
     bjb_controller_set_collection (view->priv->controller,
-                                   biji_item_get_title (to_open));
+                                   BIJI_COLLECTION (to_open));
   }
 }
 
@@ -248,106 +246,33 @@ get_note_url_from_tree_path(GtkTreePath *path, BjbMainView *self)
   return note_path ;
 }
 
-void
-action_tag_selected_items (GtkWidget *w, BjbMainView *view)
+
+GList *
+bjb_main_view_get_selected_items (BjbMainView *view)
 {
-  GList *notes = NULL;
-  GList *paths, *l;
-
-  /*  GtkTreePath */
-  paths = get_selected_paths(view);
-
-  for (l=paths ; l != NULL ; l=l->next)
-  {
-    gchar *url = get_note_url_from_tree_path (l->data, view) ;
-    notes = g_list_prepend (notes, biji_note_book_get_item_at_path
-                                 (bjb_window_base_get_book(view->priv->window),url));
-    g_free (url);
-  }
-
-  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-  bjb_note_tag_dialog_new (GTK_WINDOW (view->priv->window), notes);
-  g_list_free (notes);
-}
-
-gboolean
-bjb_main_view_get_selected_items_color (BjbMainView *view, GdkRGBA *color)
-{
-  GList *paths;
+  GList *l, *paths, *result = NULL;
   gchar *url;
   BijiItem *item;
 
   /*  GtkTreePath */
-  paths = get_selected_paths(view);
-  url = get_note_url_from_tree_path (paths->data, view) ;
-  item = biji_note_book_get_item_at_path (bjb_window_base_get_book(view->priv->window), url);
-  g_free (url);
-  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
+  paths = get_selected_paths (view);
 
-  if (BIJI_IS_NOTE_OBJ (item))
-    return biji_note_obj_get_rgba (BIJI_NOTE_OBJ (item), color);
 
-  return FALSE;
-}
-
-void
-action_color_selected_items (GtkWidget *w, BjbMainView *view)
-{
-  GList *items = NULL ;
-  GList *paths, *l;
-
-  GdkRGBA color;
-  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (w), &color);
-
-  /*  GtkTreePath */
-  paths = get_selected_paths(view);
-
-  for (l=paths ; l != NULL ; l=l->next)
+  for (l=paths; l!= NULL; l=l->next)
   {
-    gchar *url = get_note_url_from_tree_path (l->data, view) ;
-    items = g_list_prepend (items, biji_note_book_get_item_at_path
-                                   (bjb_window_base_get_book(view->priv->window),url));
+    url = get_note_url_from_tree_path (l->data, view);
+    item = biji_note_book_get_item_at_path (
+              bjb_window_base_get_book (view->priv->window), url);
+    if (BIJI_IS_ITEM (item))
+      result = g_list_prepend (result, item);
+
     g_free (url);
   }
 
   g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-
-  for (l=items ; l != NULL ; l=l->next)
-  {
-    if (BIJI_IS_NOTE_OBJ (l->data))
-      biji_note_obj_set_rgba (BIJI_NOTE_OBJ (l->data), &color);
-  }
-
-  g_list_free (items);
+  return result;
 }
 
-void
-action_delete_selected_items (GtkWidget *w, BjbMainView *view)
-{
-  GList *items = NULL;
-  GList *paths, *l;
-
-  /*  GtkTreePath */
-  paths = get_selected_paths(view);
-
-  for (l=paths ; l != NULL ; l=l->next)
-  {
-    gchar *url = get_note_url_from_tree_path (l->data, view) ;
-    items = g_list_prepend (items, biji_note_book_get_item_at_path
-                               (bjb_window_base_get_book(view->priv->window),url));
-    g_free (url);
-  }
-
-  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-
-  for (l=items ; l != NULL ; l=l->next)
-  {
-    biji_note_book_remove_item (bjb_window_base_get_book (view->priv->window),
-                                BIJI_ITEM (l->data));
-  }
-
-  g_list_free (items);
-}
 
 static void
 on_selection_mode_changed_cb (BjbMainView *self)
@@ -411,19 +336,21 @@ on_item_activated (GdMainView        * gd,
   BijiNoteBook * book ;
   BijiItem     * to_open ;
   GtkTreeIter    iter ;
-  gchar        * note_path ;
+  gchar        * item_path ;
   GtkTreeModel * model ;
 
-  /* Get Note Path */
+  /* Get Item Path */
   model = gd_main_view_get_model (gd);
   gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) path);
-  gtk_tree_model_get (model, &iter, GD_MAIN_COLUMN_URI, &note_path,-1);
+  gtk_tree_model_get (model, &iter, GD_MAIN_COLUMN_URI, &item_path,-1);
 
-  /* Switch to that note */
+  /* Switch to that item */
   book = bjb_window_base_get_book (view->priv->window); 
-  to_open = biji_note_book_get_item_at_path (book, note_path);
-  g_free (note_path);
-  switch_to_note (view, to_open); 
+  to_open = biji_note_book_get_item_at_path (book, item_path);
+  g_free (item_path);
+
+  if (to_open)
+    switch_to_item (view, to_open);
 
   return FALSE ;
 }
@@ -456,7 +383,7 @@ on_drag_data_received (GtkWidget        *widget,
 
       /* FIXME Text is guchar utf 8, conversion to perform */
       book =  bjb_window_base_get_book (self->priv->window); 
-      ret = biji_note_book_new_note_with_text (book, (gchar*) text);
+      ret = biji_note_book_note_new (book, (gchar*) text);
       switch_to_note_view (self, ret); // maybe AFTER drag finish?
 
       g_free (text);
@@ -468,14 +395,12 @@ on_drag_data_received (GtkWidget        *widget,
   gtk_drag_finish (context, FALSE, FALSE, time);
 }
 
+
 void
 bjb_main_view_connect_signals (BjbMainView *self)
 {
   BjbMainViewPriv *priv = self->priv;
 
-  bjb_controller_connect (priv->controller);
-  bjb_search_toolbar_connect (priv->search_bar);
-
   if (priv->view_selection_changed == 0)
     priv->view_selection_changed = g_signal_connect_swapped
                                   (priv->view,
@@ -503,7 +428,6 @@ bjb_main_view_constructed(GObject *o)
   GtkBox               *vbox; //self, too
   BjbMainViewPriv      *priv;
   GtkOverlay           *overlay;
-  GdRevealer           *revealer;
 
   G_OBJECT_CLASS (bjb_main_view_parent_class)->constructed(G_OBJECT(o));
 
@@ -517,11 +441,6 @@ bjb_main_view_constructed(GObject *o)
 
   priv->view = gd_main_view_new (DEFAULT_VIEW);
 
-  /* Search entry toolbar */
-  priv->search_bar = bjb_search_toolbar_new (priv->window, priv->controller);
-  revealer = bjb_search_toolbar_get_revealer (priv->search_bar);
-  gtk_box_pack_start (vbox, GTK_WIDGET (revealer), FALSE, FALSE, 0);
-
   /* Main view */
   overlay = GTK_OVERLAY (gtk_overlay_new ());
 
@@ -531,6 +450,10 @@ bjb_main_view_constructed(GObject *o)
   gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (priv->view));
   gtk_box_pack_start (vbox, GTK_WIDGET (overlay), TRUE, TRUE, 0);
 
+  /* Load more */
+  priv->load_more = bjb_load_more_button_new (priv->controller);
+  gtk_box_pack_start (vbox, priv->load_more, FALSE, FALSE, 0);
+
   /* Selection Panel */
   priv->select_bar = bjb_selection_toolbar_new (priv->view, self);
   gtk_overlay_add_overlay (overlay, GTK_WIDGET (priv->select_bar));
@@ -611,23 +534,9 @@ bjb_main_view_update_model (BjbMainView *self)
   gd_main_view_set_model (priv->view, bjb_controller_get_model (priv->controller));
 }
 
-BjbSearchToolbar *
-bjb_main_view_get_search_toolbar (BjbMainView *view)
-{
-  g_return_val_if_fail (BJB_IS_MAIN_VIEW (view), NULL);
+/* interface for notes view (GdMainView)
+ * TODO - BjbMainView should rather be a GdMainView */
 
-  return view->priv->search_bar;
-}
-
-gpointer
-bjb_main_view_get_main_toolbar (BjbMainView *view)
-{
-  g_return_val_if_fail (BJB_IS_MAIN_VIEW (view), NULL);
-
-  return (gpointer) view->priv->main_toolbar;
-}
-
-/* interface for notes view (GdMainView) */
 gboolean
 bjb_main_view_get_selection_mode (BjbMainView *self)
 {
@@ -652,8 +561,3 @@ bjb_main_view_set_view_type (BjbMainView *view, GdMainViewType type)
   gd_main_view_set_view_type (view->priv->view, type);
 }
 
-GList *
-bjb_main_view_get_selection (BjbMainView *view)
-{
-  return gd_main_view_get_selection (view->priv->view);
-}
