@@ -26,6 +26,7 @@
 /* Model for tree view */
 enum {
   COL_SELECTION,
+  COL_URN,
   COL_TAG_NAME,
   N_COLUMNS
 };
@@ -64,7 +65,8 @@ struct _BjbNoteTagDialogPrivate
   // tmp when a new tag added
   gchar *tag_to_scroll_to;
 
-  // tmp for convenience, when a tag is toggled
+  // for convenience, when a tag is toggled
+  // stores the collection urn
   gchar *toggled_collection;
 
 };
@@ -74,7 +76,7 @@ struct _BjbNoteTagDialogPrivate
 G_DEFINE_TYPE (BjbNoteTagDialog, bjb_note_tag_dialog, GTK_TYPE_DIALOG);
 
 static void
-append_tag (gchar *tag, BjbNoteTagDialog *self)
+append_collection (BijiTrackerInfoSet *set, BjbNoteTagDialog *self)
 {
   BjbNoteTagDialogPrivate *priv = self->priv;
 
@@ -83,12 +85,12 @@ append_tag (gchar *tag, BjbNoteTagDialog *self)
   GList *l;
 
   gtk_list_store_append (priv->store, &iter);
-  item_has_tag = biji_item_has_collection (priv->items->data, tag);
+  item_has_tag = biji_item_has_collection (priv->items->data, set->title);
 
   /* Check if other notes have the same */  
   for (l = priv->items; l != NULL; l = l->next)
   {
-    if (biji_item_has_collection (l->data, tag) != item_has_tag)
+    if (biji_item_has_collection (l->data, set->title) != item_has_tag)
     {
       item_has_tag = SELECTION_INCONSISTENT;
       break;
@@ -97,17 +99,22 @@ append_tag (gchar *tag, BjbNoteTagDialog *self)
 
   gtk_list_store_set (priv->store,    &iter,
                       COL_SELECTION,  item_has_tag,
-                      COL_TAG_NAME ,  tag, -1);
+                      COL_URN,        set->urn,
+                      COL_TAG_NAME ,  set->title, -1);
 }
 
 static gint
-bjb_compare_tag (gconstpointer a, gconstpointer b)
+bjb_compare_collection (gconstpointer a, gconstpointer b)
 {
-  gchar *up_a, *up_b;
-  gint retval;
 
-  up_a = g_utf8_strup (a, -1);
-  up_b = g_utf8_strup (b, -1);
+  gchar *up_a, *up_b;
+  BijiTrackerInfoSet *set_a, *set_b;
+  gint retval;
+  set_a = (BijiTrackerInfoSet *) a;
+  set_b = (BijiTrackerInfoSet *) b;
+
+  up_a = g_utf8_strup (set_a->title, -1);
+  up_b = g_utf8_strup (set_b->title, -1);
   retval = g_strcmp0 (up_a, up_b);
 
   g_free (up_a);
@@ -153,24 +160,21 @@ bjb_get_path_for_str (GtkTreeModel  *model,
 }
 
 static void
-bjb_note_tag_dialog_handle_tags (GObject *source_object,
-                                 GAsyncResult *res,
-                                 gpointer user_data)
+bjb_note_tag_dialog_handle_tags (GHashTable *result, gpointer user_data)
 {
   BjbNoteTagDialog *self = BJB_NOTE_TAG_DIALOG (user_data);
   BjbNoteTagDialogPrivate *priv = self->priv;
-  GList *collections;
+  GList *tracker_info;
 
   if (priv->collections)
     g_hash_table_destroy (priv->collections);
 
-  priv->collections = biji_get_all_collections_finish (source_object, res);
+  priv->collections = result;
 
-  collections = g_hash_table_get_keys (priv->collections);
-  collections = g_list_sort (collections, bjb_compare_tag);
-
-  g_list_foreach (collections, (GFunc) append_tag, self);
-  g_list_free (collections);
+  tracker_info = g_hash_table_get_values (priv->collections);
+  tracker_info = g_list_sort (tracker_info, bjb_compare_collection);
+  g_list_foreach (tracker_info, (GFunc) append_collection, self);
+  g_list_free (tracker_info);
 
   /* If a new tag was added, scroll & free */
   if (priv->tag_to_scroll_to)
@@ -191,28 +195,27 @@ bjb_note_tag_dialog_handle_tags (GObject *source_object,
 static void
 update_collections_model_async (BjbNoteTagDialog *self)
 {
+  BijiNoteBook *book;
+
+  book = bjb_window_base_get_book (GTK_WIDGET (self->priv->window));
   gtk_list_store_clear (self->priv->store);
-  biji_get_all_collections_async (bjb_note_tag_dialog_handle_tags, self);
+  biji_get_all_collections_async (book, bjb_note_tag_dialog_handle_tags, self);
 }
 
 /* Libbiji handles tracker & saving */
 static void
-note_dialog_add_tag (gpointer iter, gpointer collection)
+note_dialog_add_collection (gpointer iter, gpointer user_data)
 {
-  biji_item_add_collection (BIJI_ITEM (iter), (gchar*) collection, TRUE);
+  g_return_if_fail (BIJI_IS_COLLECTION (user_data));
+  biji_item_add_collection (iter, user_data, NULL);
 }
 
+
 static void
-note_dialog_remove_tag (gpointer iter, gpointer user_data)
+note_dialog_remove_collection (gpointer iter, gpointer user_data)
 {
-  BjbNoteTagDialog *self;
-  gchar *urn;
-
-  self = user_data;
-  urn = g_hash_table_lookup (self->priv->collections,
-                             self->priv->toggled_collection);
-
-  biji_item_remove_collection (BIJI_ITEM (iter), self->priv->toggled_collection, urn);
+  g_return_if_fail (BIJI_IS_COLLECTION (user_data));
+  biji_item_remove_collection (iter, user_data);
 }
 
 static void
@@ -228,23 +231,31 @@ on_tag_toggled (GtkCellRendererToggle *cell,
   gint toggle_item;
   gint *column;
   gchar *tag;
+  BijiNoteBook *book;
+  BijiItem *collection;
 
   column = g_object_get_data (G_OBJECT (cell), "column");
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_model_get (model, &iter, column, &toggle_item, -1);
-  gtk_tree_model_get (model, &iter,COL_TAG_NAME, &tag,-1);
+  gtk_tree_model_get (model, &iter, COL_URN, &tag, -1);
 
   priv->toggled_collection = tag;
+  book = bjb_window_base_get_book (GTK_WIDGET (self->priv->window));
+  collection = biji_note_book_get_item_at_path (book, tag);
 
-  if (toggle_item == SELECTION_INCONSISTENT || toggle_item == SELECTION_FALSE)
+  if (BIJI_IS_COLLECTION (collection))
   {
-    g_list_foreach (priv->items, note_dialog_add_tag, tag);
-    toggle_item = SELECTION_TRUE;
-  }
-  else
-  {
-    g_list_foreach (priv->items, note_dialog_remove_tag, self);
-    toggle_item = SELECTION_FALSE;
+    if (toggle_item == SELECTION_INCONSISTENT || toggle_item == SELECTION_FALSE)
+    {
+      g_list_foreach (priv->items, note_dialog_add_collection, collection);
+      toggle_item = SELECTION_TRUE;
+    }
+
+    else
+    {
+      g_list_foreach (priv->items, note_dialog_remove_collection, collection);
+      toggle_item = SELECTION_FALSE;
+    }
   }
 
   priv->toggled_collection = NULL;
@@ -252,26 +263,34 @@ on_tag_toggled (GtkCellRendererToggle *cell,
   gtk_tree_path_free (path);
 }
 
+
+/* If the collection with same title already existed,
+ * libbiji has to avoid creating a new one
+ * and also check before tagging items */
 static void
-on_new_collection_created_cb (gpointer user_data)
+on_new_collection_created_cb (BijiItem *coll, gpointer user_data)
 {
   BjbNoteTagDialog *self = user_data;
   BjbNoteTagDialogPrivate *priv = self->priv;
 
   priv->tag_to_scroll_to = g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->entry)));
-  g_list_foreach (priv->items, note_dialog_add_tag, priv->tag_to_scroll_to);
+  g_list_foreach (priv->items, note_dialog_add_collection, coll);
 
   update_collections_model_async (self);
   gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
 }
 
+/* Gives the title and book :
+ * the collection is created & book updated.
+ * afterward, our callback comes */
 static void
 add_new_tag (BjbNoteTagDialog *self)
 {
-  const gchar *collection = gtk_entry_get_text (GTK_ENTRY (self->priv->entry));
+  BijiNoteBook *book = bjb_window_base_get_book (GTK_WIDGET (self->priv->window));
+  const gchar *title = gtk_entry_get_text (GTK_ENTRY (self->priv->entry));
 
-  if (collection && g_utf8_strlen (collection, -1) > 0)
-    biji_create_new_collection (collection, on_new_collection_created_cb, self);
+  if (title && g_utf8_strlen (title, -1) > 0)
+    biji_create_new_collection_async (book, title, on_new_collection_created_cb, self);
 }
 
 static void
@@ -317,7 +336,12 @@ add_columns (GtkTreeView *view, BjbNoteTagDialog *self)
                                            NULL,
                                            NULL);
 
-  /* List column: tag */
+  /* URN */
+  column = gtk_tree_view_column_new ();
+  gtk_tree_view_append_column (view, column);
+
+
+  /* List column: collection title */
   column = gtk_tree_view_column_new ();
   gtk_tree_view_append_column (view, column);
 
@@ -343,12 +367,14 @@ bjb_note_tag_dialog_init (BjbNoteTagDialog *self)
   gtk_window_set_default_size (GTK_WINDOW (self),
                                BJB_NOTE_TAG_DIALOG_DEFAULT_WIDTH,
                                BJB_NOTE_TAG_DIALOG_DEFAULT_HEIGHT);
+  gtk_window_set_title (GTK_WINDOW (self), _("Collections"));
 
   g_signal_connect_swapped (self, "response",
                             G_CALLBACK (gtk_widget_destroy), self);
 
   priv->store = gtk_list_store_new (N_COLUMNS,
-                                    G_TYPE_INT,      // tag active
+                                    G_TYPE_INT,      // collection is active
+                                    G_TYPE_STRING,   // collection urn
                                     G_TYPE_STRING);  // collection title 
 }
 
@@ -370,7 +396,7 @@ bjb_note_tag_dialog_constructed (GObject *obj)
   area = gtk_dialog_get_content_area (GTK_DIALOG (self));
   gtk_container_set_border_width (GTK_CONTAINER (area), 8);
 
-  label = gtk_label_new (_("Enter a name to create a tag"));
+  label = gtk_label_new (_("Enter a name to create a collection"));
   gtk_box_pack_start (GTK_BOX (area), label, FALSE, FALSE, 2);
 
   /* New Tag */
@@ -380,7 +406,7 @@ bjb_note_tag_dialog_constructed (GObject *obj)
   self->priv->entry = gtk_entry_new();
   gtk_box_pack_start (GTK_BOX (hbox), self->priv->entry, TRUE, TRUE, 0);
 
-  new = gtk_button_new_with_label (_("New tag"));
+  new = gtk_button_new_with_label (_("New collection"));
   g_signal_connect_swapped (new, "clicked", G_CALLBACK (add_new_tag), self);
 
   gtk_box_pack_start (GTK_BOX (hbox), new, FALSE, FALSE, 2);
