@@ -57,13 +57,13 @@ biji_item_init (BijiItem *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, BIJI_TYPE_ITEM, BijiItemPrivate);
 }
 
-gchar *
+const gchar *
 biji_item_get_title         (BijiItem *item)
 {
   return BIJI_ITEM_GET_CLASS (item)->get_title (item);
 }
 
-gchar *
+const gchar *
 biji_item_get_uuid          (BijiItem *item)
 {
   return BIJI_ITEM_GET_CLASS (item)->get_uuid (item);
@@ -81,10 +81,18 @@ biji_item_get_emblem        (BijiItem *item)
   return BIJI_ITEM_GET_CLASS (item)->get_emblem (item);
 }
 
-glong
-biji_item_get_last_change    (BijiItem *item)
+GdkPixbuf *
+biji_item_get_pristine        (BijiItem *item)
 {
-  return BIJI_ITEM_GET_CLASS (item)->get_change_sec (item);
+  return BIJI_ITEM_GET_CLASS (item)->get_pristine (item);
+}
+
+
+
+gint64
+biji_item_get_mtime           (BijiItem *item)
+{
+  return BIJI_ITEM_GET_CLASS (item)->get_mtime (item);
 }
 
 gboolean
@@ -99,12 +107,12 @@ biji_item_has_collection      (BijiItem *item, gchar *coll)
   return BIJI_ITEM_GET_CLASS (item)->has_collection (item, coll);
 }
 
-gboolean         biji_item_add_collection      (BijiItem *item, gchar *coll, gboolean on_user_action)
+gboolean         biji_item_add_collection      (BijiItem *item, BijiItem *coll, gchar *title)
 {
-  return BIJI_ITEM_GET_CLASS (item)->add_collection (item, coll, on_user_action);
+  return BIJI_ITEM_GET_CLASS (item)->add_collection (item, coll, title);
 }
 
-gboolean         biji_item_remove_collection   (BijiItem *item, gchar *coll, gchar *urn)
+gboolean         biji_item_remove_collection   (BijiItem *item, BijiItem *collection)
 {
-  return BIJI_ITEM_GET_CLASS (item)->remove_collection (item, coll, urn);
+  return BIJI_ITEM_GET_CLASS (item)->remove_collection (item, collection);
 }
