@@ -60,10 +60,10 @@ biji_note_book_init (BijiNoteBook *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, BIJI_TYPE_NOTE_BOOK,
                                             BijiNoteBookPrivate);
 
-  /* Note path is key for table. It's freed by note itself */
+  /* Item path is key for table */
   self->priv->items = g_hash_table_new_full (g_str_hash,
                                              g_str_equal,
-                                             g_free,
+                                             NULL,
                                              g_object_unref);
 }
 
@@ -147,7 +147,7 @@ title_is_unique (BijiNoteBook *book, gchar *title)
 
 /* If title not unique, add sufffix "n", starting with 2, until ok */
 gchar *
-biji_note_book_get_unique_title (BijiNoteBook *book, gchar *title)
+biji_note_book_get_unique_title (BijiNoteBook *book, const gchar *title)
 {
   if (!book)
     return g_strdup (title);
@@ -179,6 +179,14 @@ biji_note_book_notify_changed (BijiNoteBook            *book,
   return FALSE;
 }
 
+/* TODO : use the same for note, put everything there
+ * rather calling a func */
+static void
+on_item_deleted_cb (BijiItem *item, BijiNoteBook *book)
+{
+  biji_note_book_remove_item (book, item);
+}
+
 void
 book_on_note_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
 {
@@ -186,45 +194,27 @@ book_on_note_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
 }
 
 static void
-book_on_note_color_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
+book_on_item_icon_changed_cb (BijiNoteObj *note, BijiNoteBook *book)
 {
-  biji_note_book_notify_changed (book, BIJI_BOOK_NOTE_COLORED, BIJI_ITEM (note));
+  biji_note_book_notify_changed (book, BIJI_BOOK_ITEM_ICON_CHANGED, BIJI_ITEM (note));
 }
 
 static void
 _biji_note_book_add_one_note (BijiNoteBook *book, BijiNoteObj *note)
 {
-  GList *collections, *l;
   g_return_if_fail (BIJI_IS_NOTE_OBJ (note));
 
   _biji_note_obj_set_book (note, (gpointer) book);
 
   /* Add it to the list */
   g_hash_table_insert (book->priv->items,
-                       biji_item_get_uuid (BIJI_ITEM (note)), note);
-
-  /* Check for new collections */
-  collections = biji_note_obj_get_collections (note);
-
-  for (l = collections ; l != NULL; l = l->next)
-  {
-    BijiCollection *collection;
-
-    collection = g_hash_table_lookup (book->priv->items, l->data);
-
-    if (!collection)
-    {
-      collection = biji_collection_new ((gchar*) l->data);
-      g_hash_table_insert (book->priv->items,
-                           biji_item_get_uuid (BIJI_ITEM (collection)),
-                           collection);
-    }
-  }
+                       (gpointer) biji_item_get_uuid (BIJI_ITEM (note)),
+                       note);
 
   /* Notify */
   g_signal_connect (note, "changed", G_CALLBACK (book_on_note_changed_cb), book);
   g_signal_connect (note, "renamed", G_CALLBACK (book_on_note_changed_cb), book);
-  g_signal_connect (note, "color-changed", G_CALLBACK (book_on_note_color_changed_cb), book);
+  g_signal_connect (note, "color-changed", G_CALLBACK (book_on_item_icon_changed_cb), book);
 }
 
 #define ATTRIBUTES_FOR_NOTEBOOK "standard::content-type,standard::name"
@@ -249,6 +239,46 @@ release_enum_cb (GObject *source, GAsyncResult *res, gpointer user_data)
   g_object_unref (source);
 }
 
+static void
+create_collection_if_needed (gpointer key,
+                             gpointer value,
+                             gpointer user_data)
+{
+  BijiNoteBook *book = BIJI_NOTE_BOOK (user_data);
+  BijiTrackerInfoSet *set = value;
+  BijiCollection *collection;
+
+  collection = g_hash_table_lookup (book->priv->items, key);
+
+  if (!collection)
+  {
+    collection = biji_collection_new (G_OBJECT (book), key, set->title, set->mtime);
+
+    g_hash_table_insert (book->priv->items,
+                         g_strdup (key),
+                         collection);
+
+    g_signal_connect (collection, "deleted",
+                      G_CALLBACK (on_item_deleted_cb), book);
+    g_signal_connect (collection , "icon-changed",
+                      G_CALLBACK (book_on_item_icon_changed_cb), book);
+  }
+
+  /* InfoSet are freed per g_hash_table_destroy thanks to below caller */
+}
+
+static void
+load_book_finish (GHashTable *collections,
+                  gpointer user_data)
+{
+  BijiNoteBook *self = BIJI_NOTE_BOOK (user_data);
+
+  g_hash_table_foreach (collections, create_collection_if_needed, user_data);
+  g_hash_table_destroy (collections);
+
+  biji_note_book_notify_changed (self, BIJI_BOOK_MASS_CHANGE, NULL);
+}
+
 static void
 enumerate_next_files_ready_cb (GObject *source,
                                GAsyncResult *res,
@@ -298,7 +328,9 @@ enumerate_next_files_ready_cb (GObject *source,
   g_free (base_path);
   g_list_free_full (files, g_object_unref);
 
-  biji_note_book_notify_changed (self, BIJI_BOOK_MASS_CHANGE, NULL);
+  /* Now we have all notes,
+   * load the collections and we're good to notify loading done */
+  biji_get_all_collections_async (self, load_book_finish, self);
 }
 
 static void
@@ -397,7 +429,7 @@ biji_note_book_remove_item (BijiNoteBook *book, BijiItem *item)
   g_return_val_if_fail (BIJI_IS_ITEM      (item), FALSE);
 
   BijiItem *to_delete = NULL;
-  gchar *path;
+  const gchar *path;
   gboolean retval = FALSE;
 
   path = biji_item_get_uuid (item);
@@ -417,21 +449,40 @@ biji_note_book_remove_item (BijiNoteBook *book, BijiItem *item)
     retval = TRUE;
   }
 
-  g_free (path);
   return retval;
 }
 
-/* Notes collection */
-void
-biji_note_book_append_new_note (BijiNoteBook *book, BijiNoteObj *note, gboolean notify)
+gboolean
+biji_note_book_add_item (BijiNoteBook *book, BijiItem *item, gboolean notify)
 {
-  g_return_if_fail (BIJI_IS_NOTE_BOOK (book));
-  g_return_if_fail (BIJI_IS_NOTE_OBJ (note));
+  g_return_val_if_fail (BIJI_IS_NOTE_BOOK (book), FALSE);
+  g_return_val_if_fail (BIJI_IS_ITEM (item), FALSE);
 
-  _biji_note_book_add_one_note (book,note);
+  const gchar *uid;
+  gboolean retval = TRUE;
 
-  if (notify)
-    biji_note_book_notify_changed (book, BIJI_BOOK_ITEM_ADDED, BIJI_ITEM (note));
+  uid = biji_item_get_uuid (item);
+
+  if (g_hash_table_lookup (book->priv->items, uid))
+    retval = FALSE;
+
+  else if (BIJI_IS_NOTE_OBJ (item))
+    _biji_note_book_add_one_note (book, BIJI_NOTE_OBJ (item));
+
+  else if (BIJI_IS_COLLECTION (item))
+  {
+    g_hash_table_insert (book->priv->items,
+                         (gpointer) biji_item_get_uuid (item),
+                         item);
+
+    g_signal_connect (item, "deleted",
+                      G_CALLBACK (on_item_deleted_cb), book);
+  }
+
+  if (retval && notify)
+    biji_note_book_notify_changed (book, BIJI_BOOK_ITEM_ADDED, item);
+
+  return retval;
 }
 
 GList *
@@ -441,9 +492,9 @@ biji_note_book_get_items (BijiNoteBook *book)
 }
 
 BijiItem *
-biji_note_book_get_item_at_path (BijiNoteBook *book, gchar *path)
+biji_note_book_get_item_at_path (BijiNoteBook *book, const gchar *path)
 {
-  return g_hash_table_lookup (book->priv->items, path);
+  return g_hash_table_lookup (book->priv->items, (gconstpointer) path);
 }
 
 BijiNoteBook *
@@ -502,53 +553,34 @@ get_note_skeleton (BijiNoteBook *book)
   return ret;
 }
 
-/* TODO : different New notes shall call a common
- * biji_note_obj_new with different properties : path mandatory,
- * optional title, raw_text, html, ... */
-
-BijiNoteObj*
-biji_note_book_get_new_note_from_string (BijiNoteBook *book,
-                                         gchar *title)
-{
-  BijiNoteObj *ret = get_note_skeleton (book);
-
-  /* Note will copy title
-   * We do NOT sanitize here because blank title is allowed ("initial") */
-  if (title && g_strcmp0 (title, "") !=0)
-    biji_note_obj_set_title (ret, title);
-
-  biji_note_obj_save_note (ret);
-  biji_note_book_append_new_note (book, ret, TRUE);
-
-  return ret;
-}
-
 static char*
 wrap_note_content (char *content)
 {
   return g_strdup_printf("<html xmlns=\"http://www.w3.org/1999/xhtml\"><body>%s</body></html>", content);
 }
 
-
 BijiNoteObj *
-biji_note_book_new_note_with_text (BijiNoteBook *book,
-                                   gchar *plain_text)
+biji_note_book_note_new           (BijiNoteBook *book, gchar *str)
 {
   BijiNoteObj *ret = get_note_skeleton (book);
-  gchar *unique_title = biji_note_book_get_unique_title (book, DEFAULT_NOTE_TITLE);
-  gchar *html;
 
-  /* Note will copy title, raw_text and html strings */
-  biji_note_obj_set_title (ret, unique_title);
-  g_free (unique_title);
+  if (str)
+  {
+    gchar *unique, *html;
 
-  biji_note_obj_set_raw_text (ret, plain_text);
-  html = wrap_note_content (plain_text);
-  biji_note_obj_set_html_content (ret, html);
-  g_free (html);
+    unique = biji_note_book_get_unique_title (book, str);
+    html = wrap_note_content (str);
+
+    biji_note_obj_set_title (ret, unique);
+    biji_note_obj_set_raw_text (ret, str);
+    biji_note_obj_set_html_content (ret, html);
+
+    g_free (unique);
+    g_free (html);
+  }
 
   biji_note_obj_save_note (ret);
-  biji_note_book_append_new_note (book, ret, TRUE);
+  biji_note_book_add_item (book, BIJI_ITEM (ret), TRUE);
 
   return ret;
 }
