@@ -7,17 +7,15 @@
 
 G_BEGIN_DECLS
 
-#define DEFAULT_NOTE_TITLE "New Note"
-
 /* The flag tells if view should reload the whole model or not */
 typedef enum
 {
   BIJI_BOOK_CHANGE_FLAG,
-  BIJI_BOOK_MASS_CHANGE,   // Startup, mass import.. rather rebuild the whole.
-  BIJI_BOOK_ITEM_ADDED,    // Single item added
-  BIJI_BOOK_ITEM_TRASHED,  // Single item trashed
-  BIJI_BOOK_NOTE_AMENDED,  // Single note amended (title, content)
-  BIJI_BOOK_NOTE_COLORED,  // Single note color
+  BIJI_BOOK_MASS_CHANGE,        // Startup, mass import.. rather rebuild the whole.
+  BIJI_BOOK_ITEM_ADDED,         // Single item added
+  BIJI_BOOK_ITEM_TRASHED,       // Single item trashed
+  BIJI_BOOK_ITEM_ICON_CHANGED,  // Single item icon
+  BIJI_BOOK_NOTE_AMENDED,       // Single note amended (title, content)
 } BijiNoteBookChangeFlag;
 
 #define BIJI_TYPE_NOTE_BOOK             (biji_note_book_get_type ())
@@ -47,9 +45,9 @@ GType biji_note_book_get_type (void) G_GNUC_CONST;
 
 BijiNoteBook * biji_note_book_new (GFile *location);
 
-gchar * biji_note_book_get_unique_title (BijiNoteBook *book, gchar *title);
+gchar * biji_note_book_get_unique_title (BijiNoteBook *book, const gchar *title);
 
-void biji_note_book_append_new_note (BijiNoteBook *book, BijiNoteObj *note, gboolean notify);
+gboolean biji_note_book_add_item (BijiNoteBook *book, BijiItem *item, gboolean notify);
 
 gboolean biji_note_book_notify_changed (BijiNoteBook           *book,
                                         BijiNoteBookChangeFlag  flag,
@@ -57,18 +55,15 @@ gboolean biji_note_book_notify_changed (BijiNoteBook           *book,
 
 gboolean biji_note_book_remove_item (BijiNoteBook *book, BijiItem *item);
 
-BijiItem * biji_note_book_get_item_at_path (BijiNoteBook *book, gchar *path);
+BijiItem * biji_note_book_get_item_at_path (BijiNoteBook *book, const gchar *path);
 
 /* Get all items, either notes or collections
  * Free the GList, not its content */
 GList * biji_note_book_get_items (BijiNoteBook *book);
 
-/* New Notes */
 BijiNoteObj* biji_note_get_new_from_file (const gchar* tomboy_format_note_path);
 
-BijiNoteObj * biji_note_book_get_new_note_from_string (BijiNoteBook *book, gchar *title);
-
-BijiNoteObj * biji_note_book_new_note_with_text (BijiNoteBook *book, gchar *plain_text);
+BijiNoteObj * biji_note_book_note_new (BijiNoteBook *book, gchar *str);
 
 G_END_DECLS
 
