@@ -49,8 +49,7 @@ struct _BijiNoteObjPrivate
    * Emblem is smaller & just shows the color */
   GdkPixbuf             *icon;
   GdkPixbuf             *emblem;
-  gboolean              icon_needs_update;
-  gboolean              emblem_needs_update;
+  GdkPixbuf             *pristine;
 
   /* Tags 
    * In Tomboy, templates are 'system:notebook:%s' tags.*/
@@ -77,17 +76,6 @@ static GParamSpec *properties[BIJI_OBJ_PROPERTIES] = { NULL, };
 
 G_DEFINE_TYPE (BijiNoteObj, biji_note_obj, BIJI_TYPE_ITEM);
 
-/* BijiItem iface */
-static gchar     * biji_note_obj_get_title                (BijiItem *note);
-static gchar     * biji_note_obj_get_path                 (BijiItem *note);
-static GdkPixbuf * biji_note_obj_get_icon                 (BijiItem *note);
-static GdkPixbuf * biji_note_obj_get_emblem               (BijiItem *note);
-static gboolean    biji_note_obj_trash                    (BijiItem *note);
-static glong       biji_note_obj_get_last_change_date_sec (BijiItem *note);
-static gboolean    biji_note_obj_has_collection           (BijiItem *note, gchar *label);
-static gboolean    biji_note_obj_add_collection           (BijiItem *note, gchar *label, gboolean on_user_action);
-static gboolean    biji_note_obj_remove_collection        (BijiItem *note, gchar *label, gchar *urn);
-
 static void
 on_save_timeout (BijiNoteObj *self)
 {
@@ -136,6 +124,7 @@ biji_note_obj_init (BijiNoteObj *self)
   /* Icon is only computed when necessary */
   priv->icon = NULL;
   priv->emblem = NULL;
+  priv->pristine = NULL;
 
   /* Keep value unitialied, so bijiben knows to assign default color */
   priv->color = NULL;
@@ -165,8 +154,15 @@ biji_note_obj_finalize (GObject *object)
 
   g_hash_table_destroy (priv->labels);
 
-  g_clear_object (&priv->icon);
-  g_clear_object (&priv->emblem);
+  if (priv->icon)
+    g_object_unref (priv->icon);
+
+  if (priv->emblem)
+    g_object_unref (priv->emblem);
+
+  if (priv->pristine)
+    g_object_unref (priv->pristine);
+
   gdk_rgba_free (priv->color);
 
   G_OBJECT_CLASS (biji_note_obj_parent_class)->finalize (object);
@@ -221,7 +217,7 @@ biji_note_obj_get_property (GObject    *object,
   switch (property_id)
     {
     case PROP_PATH:
-      g_value_set_object (value, biji_note_id_get_path (self->priv->id));
+      g_value_set_object (value, (gpointer) biji_note_id_get_path (self->priv->id));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -229,79 +225,6 @@ biji_note_obj_get_property (GObject    *object,
     }
 }
 
-static void
-biji_note_obj_class_init (BijiNoteObjClass *klass)
-{
-  BijiItemClass*  item_class = BIJI_ITEM_CLASS (klass);
-  GObjectClass* object_class = G_OBJECT_CLASS  (klass);
-
-  object_class->constructed = biji_note_obj_constructed;
-  object_class->finalize = biji_note_obj_finalize;
-  object_class->get_property = biji_note_obj_get_property;
-  object_class->set_property = biji_note_obj_set_property;
-
-  properties[PROP_PATH] =
-    g_param_spec_string("path",
-                        "The note file",
-                        "The location where the note is stored and saved",
-                        NULL,
-                        G_PARAM_READWRITE);
-
-  g_object_class_install_properties (object_class, BIJI_OBJ_PROPERTIES, properties);
-
-  biji_obj_signals[NOTE_RENAMED] = g_signal_new ( "renamed" ,
-                                                  G_OBJECT_CLASS_TYPE (klass),
-                                                  G_SIGNAL_RUN_LAST,
-                                                  0, 
-                                                  NULL, 
-                                                  NULL,
-                                                  g_cclosure_marshal_VOID__VOID,
-                                                  G_TYPE_NONE,
-                                                  0);
-
-  biji_obj_signals[NOTE_CHANGED] = g_signal_new ( "changed" ,
-                                                  G_OBJECT_CLASS_TYPE (klass),
-                                                  G_SIGNAL_RUN_LAST,
-                                                  0, 
-                                                  NULL, 
-                                                  NULL,
-                                                  g_cclosure_marshal_VOID__VOID,
-                                                  G_TYPE_NONE,
-                                                  0);
-
-  biji_obj_signals[NOTE_COLOR_CHANGED] = g_signal_new ("color-changed" ,
-                                                  G_OBJECT_CLASS_TYPE (klass),
-                                                  G_SIGNAL_RUN_LAST,
-                                                  0, 
-                                                  NULL, 
-                                                  NULL,
-                                                  g_cclosure_marshal_VOID__VOID,
-                                                  G_TYPE_NONE,
-                                                  0);
-
-  biji_obj_signals[NOTE_DELETED] = g_signal_new ( "deleted" ,
-                                                  G_OBJECT_CLASS_TYPE (klass),
-                                                  G_SIGNAL_RUN_LAST,
-                                                  0, 
-                                                  NULL, 
-                                                  NULL,
-                                                  g_cclosure_marshal_VOID__VOID,
-                                                  G_TYPE_NONE,
-                                                  0);
-
-  g_type_class_add_private (klass, sizeof (BijiNoteObjPrivate));
-
-  /* Interface */
-  item_class->get_title = biji_note_obj_get_title;
-  item_class->get_uuid = biji_note_obj_get_path;
-  item_class->get_icon = biji_note_obj_get_icon;
-  item_class->get_emblem = biji_note_obj_get_emblem;
-  item_class->get_change_sec = biji_note_obj_get_last_change_date_sec;
-  item_class->trash = biji_note_obj_trash;
-  item_class->has_collection = biji_note_obj_has_collection;
-  item_class->add_collection = biji_note_obj_add_collection;
-  item_class->remove_collection = biji_note_obj_remove_collection;
-}
 
 BijiNoteObj *
 biji_note_obj_new_from_path (const gchar *path)
@@ -421,7 +344,7 @@ biji_note_obj_trash (BijiItem *item)
   return result;
 }
 
-static gchar *
+static const gchar *
 biji_note_obj_get_path (BijiItem *item)
 {
   g_return_val_if_fail (BIJI_IS_NOTE_OBJ (item), NULL);
@@ -436,7 +359,7 @@ BijiNoteID* note_get_id(BijiNoteObj* n)
   return n->priv->id;
 }
 
-static gchar *
+static const gchar *
 biji_note_obj_get_title (BijiItem *note)
 {
   g_return_val_if_fail (BIJI_IS_NOTE_OBJ (note), NULL);
@@ -499,12 +422,13 @@ biji_note_obj_set_last_change_date (BijiNoteObj* n,gchar* date)
   return biji_note_id_set_last_change_date (n->priv->id,date);
 }
 
-static glong
-biji_note_obj_get_last_change_date_sec (BijiItem *item)
-{
-  BijiNoteObj *n = BIJI_NOTE_OBJ (item);
 
-  return biji_note_id_get_last_change_date_sec(note_get_id(n));
+static gint64
+biji_note_obj_get_mtime (BijiItem *note)
+{
+  g_return_val_if_fail (BIJI_IS_NOTE_OBJ (note), 0);
+
+  return biji_note_id_get_mtime (BIJI_NOTE_OBJ (note)->priv->id);
 }
 
 gchar *
@@ -538,17 +462,22 @@ biji_note_obj_set_note_create_date (BijiNoteObj* n,gchar *date)
   return biji_note_id_set_create_date (n->priv->id, date);
 }
 
+static void
+biji_note_obj_clear_icons (BijiNoteObj *note)
+{
+  g_clear_pointer (&note->priv->icon, g_object_unref);
+  g_clear_pointer (&note->priv->emblem, g_object_unref);
+  g_clear_pointer (&note->priv->pristine, g_object_unref);
+}
+
 static void
 biji_note_obj_set_rgba_internal (BijiNoteObj *n, GdkRGBA *rgba)
 {
   n->priv->color = gdk_rgba_copy(rgba);
-  n->priv->icon_needs_update = TRUE;
-  n->priv->emblem_needs_update = TRUE;
 
   g_signal_emit (G_OBJECT (n), biji_obj_signals[NOTE_COLOR_CHANGED],0);
 }
 
-
 void
 biji_note_obj_set_rgba (BijiNoteObj *n, GdkRGBA *rgba)
 {
@@ -558,6 +487,7 @@ biji_note_obj_set_rgba (BijiNoteObj *n, GdkRGBA *rgba)
   else if (!gdk_rgba_equal (n->priv->color,rgba))
   {
     gdk_rgba_free (n->priv->color);
+    biji_note_obj_clear_icons (n);
     biji_note_obj_set_rgba_internal (n, rgba);
 
     biji_note_id_set_last_metadata_change_date_now (n->priv->id);
@@ -580,23 +510,6 @@ biji_note_obj_get_rgba(BijiNoteObj *n,
   return FALSE;
 }
 
-static void
-biji_note_obj_clear_icons (BijiNoteObj *note)
-{
-  if (note->priv->icon)
-  {
-    g_clear_object (&note->priv->icon);
-    note->priv->icon = NULL ;
-  }
-
-  if (note->priv->emblem)
-  {
-    g_clear_object (&note->priv->emblem);
-    note->priv->emblem = NULL ;
-  }
-
-  g_signal_emit (G_OBJECT (note), biji_obj_signals[NOTE_CHANGED],0);
-}
 
 void biji_note_obj_set_raw_text (BijiNoteObj *note, gchar *plain_text)
 {
@@ -605,6 +518,7 @@ void biji_note_obj_set_raw_text (BijiNoteObj *note, gchar *plain_text)
 
   note->priv->raw_text = g_strdup (plain_text);
   biji_note_obj_clear_icons (note);
+  g_signal_emit (note, biji_obj_signals[NOTE_CHANGED],0);
 }
 
 GList *
@@ -626,21 +540,41 @@ biji_note_obj_has_collection (BijiItem *item, gchar *label)
   return FALSE;
 }
 
-gboolean
-biji_note_obj_add_collection (BijiItem *item, gchar *label, gboolean on_user_action_cb)
+
+static void
+_biji_collection_refresh (gboolean query_result,
+                          gpointer coll)
 {
+  g_return_if_fail (BIJI_IS_COLLECTION (coll));
+
+  if (query_result)
+    biji_collection_refresh (BIJI_COLLECTION (coll));
+}
+
+
+/*static */ gboolean
+biji_note_obj_add_collection (BijiItem *item,
+                              BijiItem *collection,
+                              gchar    *title)
+{
+  BijiNoteObj *note;
+  gchar *label = title;
+
   g_return_val_if_fail (BIJI_IS_NOTE_OBJ (item), FALSE);
-  g_return_val_if_fail (label != NULL, FALSE);
-  g_return_val_if_fail (!biji_note_obj_has_collection (item, label), FALSE);
+  note = BIJI_NOTE_OBJ (item);
 
-  BijiNoteObj *note = BIJI_NOTE_OBJ (item);
-  gchar *tag = g_strdup (label);
+  if (BIJI_IS_COLLECTION (collection))
+    label = (gchar*) biji_item_get_title (collection);
 
-  g_hash_table_add (note->priv->labels, tag);
+  if (biji_note_obj_has_collection (item, label))
+    return FALSE;
 
-  if (on_user_action_cb)
+  g_hash_table_add (note->priv->labels, g_strdup (label));
+
+  if (BIJI_IS_COLLECTION (collection))
   {
-    biji_push_existing_collection_to_note (note, tag); // Tracker
+    biji_push_existing_collection_to_note (
+      note, label, _biji_collection_refresh, collection); // Tracker
     biji_note_id_set_last_metadata_change_date_now (note->priv->id);
     biji_note_obj_save_note (note);
   }
@@ -648,15 +582,19 @@ biji_note_obj_add_collection (BijiItem *item, gchar *label, gboolean on_user_act
   return TRUE;
 }
 
+
 gboolean
-biji_note_obj_remove_collection (BijiItem *item, gchar *label, gchar *urn)
+biji_note_obj_remove_collection (BijiItem *item, BijiItem *collection)
 {
   g_return_val_if_fail (BIJI_IS_NOTE_OBJ (item), FALSE);
+  g_return_val_if_fail (BIJI_IS_COLLECTION (collection), FALSE);
+
   BijiNoteObj *note = BIJI_NOTE_OBJ (item);
 
-  if (g_hash_table_remove (note->priv->labels, label))
+  if (g_hash_table_remove (note->priv->labels, biji_item_get_title (collection)))
   {
-    biji_remove_collection_from_note (note, urn); // tracker.
+    biji_remove_collection_from_note (
+      note, collection, _biji_collection_refresh, collection); // tracker.
     biji_note_id_set_last_metadata_change_date_now (note->priv->id);
     biji_note_obj_save_note (note);
     return TRUE;
@@ -711,7 +649,8 @@ biji_note_obj_get_icon_file (BijiNoteObj *note)
 {
   g_return_val_if_fail (BIJI_IS_NOTE_OBJ (note), NULL);
 
-  gchar *uuid, *basename, *filename;
+  const gchar *uuid;
+  gchar *basename, *filename;
 
   uuid = biji_note_id_get_uuid (note->priv->id);
   basename = biji_str_mass_replace (uuid, ".note", ".png", NULL);
@@ -721,7 +660,6 @@ biji_note_obj_get_icon_file (BijiNoteObj *note)
                                basename,
                                NULL);
 
-  g_free (uuid);
   g_free (basename);
 
   return filename;
@@ -753,7 +691,7 @@ biji_note_obj_get_icon (BijiItem *item)
 
   BijiNoteObj *note = BIJI_NOTE_OBJ (item);
 
-  if (note->priv->icon && !note->priv->icon_needs_update)
+  if (note->priv->icon)
     return note->priv->icon;
 
   /* Create & Draw surface */
@@ -803,11 +741,45 @@ biji_note_obj_get_icon (BijiItem *item)
   note->priv->icon = gd_embed_image_in_frame (ret, "resource:///org/gnome/bijiben/thumbnail-frame.png",
                                               &frame_slice, &frame_slice);
   g_clear_object (&ret);
-  note->priv->icon_needs_update = FALSE;
 
   return note->priv->icon;
 }
 
+static GdkPixbuf *
+biji_note_obj_get_pristine (BijiItem *item)
+{
+  GdkRGBA                note_color;
+  cairo_t               *cr;
+  cairo_surface_t       *surface = NULL;
+  BijiNoteObj           *note = BIJI_NOTE_OBJ (item);
+
+  if (note->priv->pristine)
+    return note->priv->pristine;
+
+  /* Create & Draw surface */
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+                                        BIJI_EMBLEM_WIDTH,
+                                        BIJI_EMBLEM_HEIGHT) ;
+  cr = cairo_create (surface);
+
+  /* Background */
+  cairo_rectangle (cr, 0, 0, BIJI_EMBLEM_WIDTH, BIJI_EMBLEM_HEIGHT);
+  if (biji_note_obj_get_rgba (note, &note_color))
+    gdk_cairo_set_source_rgba (cr, &note_color);
+
+  cairo_fill (cr);
+  cairo_destroy (cr);
+
+  note->priv->pristine = gdk_pixbuf_get_from_surface (surface,
+                                                      0, 0,
+                                                      BIJI_EMBLEM_WIDTH,
+                                                      BIJI_EMBLEM_HEIGHT);
+
+  cairo_surface_destroy (surface);
+
+  return note->priv->pristine;
+}
+
 static GdkPixbuf *
 biji_note_obj_get_emblem (BijiItem *item)
 {
@@ -816,7 +788,7 @@ biji_note_obj_get_emblem (BijiItem *item)
   cairo_surface_t       *surface = NULL;
   BijiNoteObj           *note = BIJI_NOTE_OBJ (item);
 
-  if (note->priv->emblem && !note->priv->emblem_needs_update)
+  if (note->priv->emblem)
     return note->priv->emblem;
 
   /* Create & Draw surface */
@@ -846,7 +818,6 @@ biji_note_obj_get_emblem (BijiItem *item)
                                                     BIJI_EMBLEM_HEIGHT);
 
   cairo_surface_destroy (surface);
-  note->priv->emblem_needs_update = FALSE;
 
   return note->priv->emblem;
 }
@@ -1046,3 +1017,82 @@ void biji_note_obj_editor_paste (BijiNoteObj *note)
     biji_webkit_editor_paste (note->priv->editor);
 }
 
+static void
+biji_note_obj_class_init (BijiNoteObjClass *klass)
+{
+  BijiItemClass*  item_class = BIJI_ITEM_CLASS (klass);
+  GObjectClass* object_class = G_OBJECT_CLASS  (klass);
+
+  object_class->constructed = biji_note_obj_constructed;
+  object_class->finalize = biji_note_obj_finalize;
+  object_class->get_property = biji_note_obj_get_property;
+  object_class->set_property = biji_note_obj_set_property;
+
+  properties[PROP_PATH] =
+    g_param_spec_string("path",
+                        "The note file",
+                        "The location where the note is stored and saved",
+                        NULL,
+                        G_PARAM_READWRITE);
+
+  g_object_class_install_properties (object_class, BIJI_OBJ_PROPERTIES, properties);
+
+  biji_obj_signals[NOTE_RENAMED] =
+    g_signal_new ("renamed",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, 
+                  NULL, 
+                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
+  biji_obj_signals[NOTE_CHANGED] =
+    g_signal_new ("changed",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, 
+                  NULL, 
+                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
+  biji_obj_signals[NOTE_COLOR_CHANGED] =
+    g_signal_new ("color-changed" ,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL,
+                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
+  biji_obj_signals[NOTE_DELETED] =
+    g_signal_new ("deleted" ,
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL,
+                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
+  g_type_class_add_private (klass, sizeof (BijiNoteObjPrivate));
+
+  /* Interface */
+  item_class->get_title = biji_note_obj_get_title;
+  item_class->get_uuid = biji_note_obj_get_path;
+  item_class->get_icon = biji_note_obj_get_icon;
+  item_class->get_emblem = biji_note_obj_get_emblem;
+  item_class->get_pristine = biji_note_obj_get_pristine;
+  item_class->get_mtime = biji_note_obj_get_mtime;
+  item_class->trash = biji_note_obj_trash;
+  item_class->has_collection = biji_note_obj_has_collection;
+  item_class->add_collection = biji_note_obj_add_collection;
+  item_class->remove_collection = biji_note_obj_remove_collection;
+}
+
