@@ -492,7 +492,7 @@ processNode (BijiLazyDeserializer *self)
     {
       norm = g_string_new (tag);
       g_string_erase (norm,0,16);
-      biji_item_add_collection (BIJI_ITEM (n), norm->str, FALSE);
+      biji_item_add_collection (BIJI_ITEM (n), NULL, norm->str);
       g_string_free (norm, TRUE);
     }
 
@@ -518,14 +518,13 @@ gboolean
 biji_lazy_deserialize_internal (BijiLazyDeserializer *self)
 {
   BijiNoteObj* n = self->priv->note;
-  gchar *path;
+  const gchar *path;
   xmlDocPtr doc;
   xmlNodePtr cur;
   xmlChar     *version; 
 
   path = biji_item_get_uuid (BIJI_ITEM (n));
   doc = xmlParseFile (path);
-  g_free (path);
 
   if (doc == NULL ) 
   {
@@ -583,7 +582,7 @@ biji_lazy_deserialize_internal (BijiLazyDeserializer *self)
 
   path = biji_item_get_uuid (BIJI_ITEM (n));
   self->priv->r = xmlNewTextReaderFilename (path);
-  g_free (path);
+
   biji_parse_file (self);
   xmlFreeDoc (doc);
 
